@@ -8,7 +8,7 @@
 # file, You can obtain one at http://mozilla.org/MPL/2.0/.
 #
 # ---
-# Author: John Egan <john@shopkick.com>
+# Author: John Egan <jwegan@gmail.com>
 
 from __future__ import absolute_import
 import __main__
@@ -32,6 +32,8 @@ import traceback
 import urllib
 import urlparse
 
+from thrift.Thrift import TType
+
 import flawless.lib.config
 from flawless.lib.data_structures import prefix_tree
 from flawless.lib.storage import DiskStorage
@@ -78,7 +80,7 @@ class FlawlessServiceBaseClass(object):
     """Contains numerous shared helper functions for the ThriftService & WebService classes"""
 
     def __init__(self,
-                 storage_cls=DiskStorage,
+                 storage_factory=DiskStorage,
                  thread_cls=threading.Thread,
                  open_process_func=subprocess.Popen,
                  smtp_client_cls=smtplib.SMTP,
@@ -93,7 +95,7 @@ class FlawlessServiceBaseClass(object):
     ############################## Parse Config Files ##############################
 
     def _read_whitelist_configs(self):
-        config_storage = self.storage_cls(partition=None)
+        config_storage = self.storage_factory(partition=None)
         config_storage.open()
 
         # Whitelists
@@ -156,6 +158,25 @@ class FlawlessServiceBaseClass(object):
 
     ############################## Traceback/Line Type Helpers ##############################
 
+    def _blame_line(self, traceback):
+        '''Figures out which line in traceback is to blame for the error.
+        Returns a 3-tuple of (ErrorKey, StackTraceEntry, [email recipients])'''
+        key = None
+        blamed_entry = None
+        email_recipients = []
+        for stack_line in traceback:
+            line_type = self._get_line_type(stack_line)
+            if line_type == api_ttypes.LineType.THIRDPARTY_WHITELIST:
+                return None, None, None, True
+            elif line_type in [api_ttypes.LineType.DEFAULT, api_ttypes.LineType.KNOWN_ERROR]:
+                filepath = self._get_basepath(stack_line.filename)
+                entry = api_ttypes.CodeIdentifier(filepath, stack_line.function_name, stack_line.text)
+                blamed_entry = entry
+                key = api_ttypes.ErrorKey(filepath, stack_line.line_number, stack_line.function_name, stack_line.text)
+                if filepath in self.watch_all_errors:
+                    email_recipients.extend(self.watch_all_errors[filepath])
+        return (key, blamed_entry, email_recipients, False)
+
     def _get_line_type(self, line):
         filepath = self._get_basepath(line.filename)
         if not filepath:
@@ -274,12 +295,12 @@ class FlawlessThriftServiceHandler(FlawlessServiceBaseClass):
         prefix = self._partition_for_ms(epoch_ms)
         with self.lock:
             if self.errors_seen is None:
-                self.errors_seen = self.storage_cls(prefix)
+                self.errors_seen = self.storage_factory(prefix)
                 self.errors_seen.open()
             elif prefix != self.errors_seen.partition:
                 # Order matters here since there can be a race condition if not done correctly
                 old_errors_seen = self.errors_seen
-                new_errors_seen = self.storage_cls(prefix)
+                new_errors_seen = self.storage_factory(prefix)
                 new_errors_seen.open()
                 self.errors_seen = new_errors_seen
                 old_errors_seen.sync()
@@ -362,27 +383,8 @@ class FlawlessThriftServiceHandler(FlawlessServiceBaseClass):
         t = self.thread_cls(target=self._record_error, args=[request])
         t.start()
 
-    def _blame_line(self, traceback):
-        '''Figures out which line in traceback is to blame for the error.
-        Returns a 3-tuple of (ErrorKey, StackTraceEntry, [email recipients])'''
-        key = None
-        blamed_entry = None
-        email_recipients = []
-        for stack_line in traceback:
-            line_type = self._get_line_type(stack_line)
-            if line_type == api_ttypes.LineType.THIRDPARTY_WHITELIST:
-                return None, None, None, True
-            elif line_type in [api_ttypes.LineType.DEFAULT, api_ttypes.LineType.KNOWN_ERROR]:
-                filepath = self._get_basepath(stack_line.filename)
-                entry = api_ttypes.CodeIdentifier(filepath, stack_line.function_name, stack_line.text)
-                blamed_entry = entry
-                key = api_ttypes.ErrorKey(filepath, stack_line.line_number, stack_line.function_name, stack_line.text)
-                if filepath in self.watch_all_errors:
-                    email_recipients.extend(self.watch_all_errors[filepath])
-        return (key, blamed_entry, email_recipients, False)
-
     def _record_error(self, request):
-        log.info("Recieved error from %s for %s" % (request.hostname, request.exception_message))
+        log.debug("Recieved error from %s for %s" % (request.hostname, request.exception_message))
 
         # Figure out which line in the stack trace is to blame for the error
         key, blamed_entry, email_recipients, was_whitelisted = self._blame_line(request.traceback)
@@ -427,8 +429,9 @@ class FlawlessThriftServiceHandler(FlawlessServiceBaseClass):
             log.info("Error %s caused by %s on %s" % (str(key), dev_email, mod_time))
 
             if not dev_email:
-                self._handle_flawless_issue("Unable to do blame for %s. You may want to consider setting "
-                                            "only_blame_filepaths_matching in your flawless.cfg " % str(key))
+                self._handle_flawless_issue("Unable to do blame for %s from %s. You may want to consider setting "
+                                            "only_blame_filepaths_matching in your flawless.cfg " %
+                                            (str(key), request.hostname))
                 err_info.email_sent = True
                 return
         # If we've already seen this error then update the error count
@@ -469,38 +472,41 @@ class FlawlessThriftServiceHandler(FlawlessServiceBaseClass):
 
         # Send email if applicable
         if send_email:
-            email_body = []
-            dev_email = self._get_email(err_info.developer_email)
-            if dev_email:
-                email_recipients.append(dev_email)
+            self._send_error_email(request, key, err_info, blamed_entry, known_entry, email_recipients)
 
-            # Add additional recipients that have registered for this error
-            if blamed_entry.filename in self.watch_only_if_blamed:
-                email_recipients.extend(self.watch_only_if_blamed[blamed_entry.filename])
-            if known_entry:
-                email_recipients.extend(known_entry.email_recipients or [])
-                email_body.append(known_entry.email_header or "")
+    def _send_error_email(self, request, err_key, err_info, blamed_entry, known_entry, email_recipients):
+        email_body = []
+        dev_email = self._get_email(err_info.developer_email)
+        if dev_email:
+            email_recipients.append(dev_email)
 
-            email_body.append(self._format_traceback(request))
-            email_body.append(
-                "<br /><br /><a href='http://%s/add_known_error?%s'>Add to whitelist</a>" %
-                (
-                    config.hostname + ":" + str(config.port),
-                    urllib.urlencode(
-                        dict(filename=key.filename, function_name=key.function_name, code_fragment=key.text)
-                    )
+        # Add additional recipients that have registered for this error
+        if blamed_entry.filename in self.watch_only_if_blamed:
+            email_recipients.extend(self.watch_only_if_blamed[blamed_entry.filename])
+        if known_entry:
+            email_recipients.extend(known_entry.email_recipients or [])
+            email_body.append(known_entry.email_header or "")
+
+        email_body.append(self._format_traceback(request))
+        email_body.append(
+            "<br /><br /><a href='%s/add_known_error?%s'>Add to whitelist</a>" %
+            (
+                config.hostname,
+                urllib.urlencode(
+                    dict(filename=err_key.filename, function_name=err_key.function_name, code_fragment=err_key.text)
                 )
             )
+        )
 
-            # Send the email
-            log.info("Sending email for %s to %s" % (str(key), ", ".join(email_recipients)))
-            self._sendmail(
-                to_addresses=email_recipients,
-                subject="Error on %s in %s" % (request.hostname, key.filename),
-                body="<br />".join([s for s in email_body if s]),
-            )
-            err_info.email_sent = True
-            self.errors_seen[key] = err_info
+        # Send the email
+        log.info("Sending email for %s to %s" % (str(err_key), ", ".join(email_recipients)))
+        self._sendmail(
+            to_addresses=email_recipients,
+            subject="Error on %s in %s" % (request.hostname, err_key.filename),
+            body="<br />".join([s for s in email_body if s]),
+        )
+        err_info.email_sent = True
+        self.errors_seen[err_key] = err_info
 
 
 ############################## WEB SERVICE ##############################
@@ -516,7 +522,7 @@ class FlawlessWebServiceHandler(FlawlessServiceBaseClass):
 
     def _get_errors_seen_for_ts(self, timestamp):
         prefix = self._partition_for_ms(int(timestamp) * 1000 if timestamp else None)
-        errors_seen = self.storage_cls(prefix)
+        errors_seen = self.storage_factory(prefix)
         errors_seen.open()
         return errors_seen
 
@@ -525,7 +531,7 @@ class FlawlessWebServiceHandler(FlawlessServiceBaseClass):
                          building_blocks=api_ttypes.CodeIdentifierList,
                          third_party_whitelist=api_ttypes.CodeIdentifierList,
                          watch_list=api_ttypes.WatchList)
-        config_storage = self.storage_cls(partition=None)
+        config_storage = self.storage_factory(partition=None)
         config_storage.open()
         current_value = config_storage[key] or class_map[key]()
         getattr(current_value, attr).append(entry)
@@ -535,9 +541,28 @@ class FlawlessWebServiceHandler(FlawlessServiceBaseClass):
         config_storage.close()
 
     def _construct_instance(self, params, cls):
+        THRIFT_SPEC_NAME_FIELD = 2
+        THRIFT_SPEC_TYPE_FIELD = 1
         init_args = inspect.getargspec(cls.__init__).args
         whitelist_attrs = [s for s in init_args if s != 'self']
-        new_entry = cls(**dict((k, params.get(k)) for k in whitelist_attrs))
+        args = dict((k, params.get(k)) for k in whitelist_attrs)
+        arg_type_map = dict()
+
+        # Build map of fields to type based on the thrift spec
+        for spec in cls.thrift_spec:
+            if not spec or len(spec) < 3:
+                continue
+            if spec[THRIFT_SPEC_TYPE_FIELD] == TType.BOOL:
+                arg_type_map[spec[THRIFT_SPEC_NAME_FIELD]] = bool
+            elif spec[THRIFT_SPEC_TYPE_FIELD] == TType.I64 or spec[THRIFT_SPEC_TYPE_FIELD] == TType.I32:
+                arg_type_map[spec[THRIFT_SPEC_NAME_FIELD]] = int
+
+        # Cast args to the appropriate type
+        for field, value in args.items():
+            if value and field in arg_type_map:
+                args[field] = arg_type_map[field](value)
+
+        new_entry = cls(**args)
         return new_entry
 
     def get_weekly_error_report(self, timestamp=None, include_known_errors=False,
@@ -752,7 +777,7 @@ class FlawlessWebServiceHandler(FlawlessServiceBaseClass):
 
     def save_remap_email(self, request):
         params = dict(urlparse.parse_qsl(request))
-        config_storage = self.storage_cls(partition=None)
+        config_storage = self.storage_factory(partition=None)
         config_storage.open()
         current_value = config_storage["email_remapping"] or api_ttypes.EmailRemapping()
         current_value.remap[params["old_email"]] = params["new_email"]
@@ -763,7 +788,7 @@ class FlawlessWebServiceHandler(FlawlessServiceBaseClass):
         return "<html><body>SUCCESS</body></html>"
 
     def view_config(self, key):
-        config_storage = self.storage_cls(partition=None)
+        config_storage = self.storage_factory(partition=None)
         config_storage.open()
         current_value = config_storage[key]
         config_storage.close()
